/////////////////////////////////////////////////////////////////////////////
// $Id: NextViewPlugin.c,v 1.5 2005/03/01 19:35:52 quenotte Exp $
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2000-2005 Quenotte  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "epgctl/mytypes.h"
#include "epgctl/debug.h"

#include "epgvbi/btdrv.h"
#include "epgvbi/winshm.h"
#include "epgvbi/btdrv.h"
#include "epgvbi/ttxdecode.h"
#include "tvsim/winshmclnt.h"

#include "nextviewplugin.h"

// TODO :
// - Grant Tuner 

TVApp CurrentTVApp;
volatile EPGACQ_BUF *pVbiBuf;

//bool WinSharedMemClient_SetStation( const char * pChanName, uint cni, uint inputSrc, uint freq );
//bool WinSharedMemClient_SetInputFreq( uint inputSrc, uint freq );

//void TtxDecode_AddPacket( const uchar * data, uint line );
//void TtxDecode_AddVpsData( const uchar * data );
//bool TtxDecode_NewVbiFrame( unsigned int frameSeqNo );

// ---------------------------------------------------------------------------
// Break up a TAB separated PI Info string into C structure
// - the given string is modified
// - warning: strings are references into the text
//   hence only valid as long as the text is not freed
//
typedef struct
{
   char       * pNetName;
   uchar        net_idx;
   time_t       start_time;
   time_t       stop_time;
   uint         pil;
   uchar        prat;
   uchar        erat;
   char       * pSound;
   bool         is_wide;
   bool         is_palplus;
   bool         is_digital;
   bool         is_encrypted;
   bool         is_live;
   bool         is_repeat;
   bool         is_subtitled;
   uchar        pdc_themes[7];
   char       * pTitle;
   char       * pDescription;
} EPG_PI;

static bool Plugin_ParsePiDescription( char * pText, EPG_PI * pi )
{
   char * pOrigText = pText;
   char * pEnd;
   struct tm t;
   int int_val, nscan, scan_pos;
   uint oldMoD;
   uint idx;

   pEnd = strchr(pText, '\t');
   if (pEnd == NULL) goto error;
   pi->pNetName = pText;
   *pEnd = 0;
   pText = pEnd + 1;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->net_idx = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%u-%u-%u\t%u:%u:00\t%n",
                         &t.tm_year, &t.tm_mon, &t.tm_mday, &t.tm_hour, &t.tm_min, &scan_pos);
   if (nscan < 5) goto error;
   t.tm_year -= 1900;
   t.tm_mon -= 1;
   t.tm_isdst = -1;
   t.tm_sec = 0;
   oldMoD = t.tm_hour*60 + t.tm_min;
   pi->start_time = mktime(&t);
   pText += scan_pos;

   nscan = sscanf(pText, "%u:%u:00\t%n", &t.tm_hour, &t.tm_min, &scan_pos);
   if (nscan < 2) goto error;
   t.tm_isdst = -1;
   t.tm_sec = 0;
   if (oldMoD < t.tm_hour*60 + t.tm_min)
      t.tm_mday += 1;
   pi->stop_time = mktime(&t);
   pText += scan_pos;

   if (strncmp(pText, "\\N\t", 3) == 0)
   {
      pi->pil = INVALID_VPS_PIL;
      pText += 3;
   }
   else
   {
      nscan = sscanf(pText, "%04u-%02u-%02u %02u:%02u:00\t%n",
                            &t.tm_year, &t.tm_mon, &t.tm_mday, &t.tm_hour, &t.tm_min, &scan_pos);
      if (nscan < 5) goto error;
      pi->pil = (t.tm_mday << 15) | (t.tm_mon << 11) | (t.tm_hour << 6) | t.tm_min;
      pText += scan_pos;
   }

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->prat = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->erat = int_val;
   pText += scan_pos;

   pEnd = strchr(pText, '\t');
   if (pEnd == NULL) goto error;
   pi->pSound = pText;
   *pEnd = 0;
   pText = pEnd + 1;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_wide = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_palplus = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_digital = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_encrypted = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_live = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_repeat = int_val;
   pText += scan_pos;

   nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
   if (nscan < 1) goto error;
   pi->is_subtitled = int_val;
   pText += scan_pos;

   for (idx = 0; idx < 7; idx++)
   {
      nscan = sscanf(pText, "%d\t%n", &int_val, &scan_pos);
      if (nscan < 1) goto error;
      pi->pdc_themes[idx] = int_val;
      pText += scan_pos;
   }

   pEnd = strchr(pText, '\t');
   if (pEnd == NULL) goto error;
   pi->pTitle = pText;
   *pEnd = 0;
   pText = pEnd + 1;

   pEnd = strchr(pText, '\n');
   if (pEnd == NULL) goto error;
   pi->pDescription = pText;
   *pEnd = 0;
   pText = pEnd + 1;

   return TRUE;

error:
   idx = pText - pOrigText;
   // undo substitutions
   for ( ; pText >= pOrigText; pText--)
   {
      if (*pText == 0)
         *pText = '\t';
   }
   debug2("TvSimu-ParsePiDescription: parse error at offset %d: %s", idx, pOrigText);
   return FALSE;
}

static bool Plugin_GetProgInfo( time_t * pStart, time_t * pStop,
                                uchar * pThemes, uint * pThemeCount,
                                uchar * pTitle, uint maxTitleLen )
{
   char * pInfo = WinSharedMemClient_GetProgInfo();
   EPG_PI pi;
   int themeIdx;
   bool result = FALSE;

   if (pInfo != NULL)
   {
      if ( Plugin_ParsePiDescription(pInfo, &pi) )
      {
         if ((pTitle != NULL) && (maxTitleLen > 0))
         {
            strncpy(pTitle, (char *) pi.pTitle, maxTitleLen);
            pTitle[maxTitleLen - 1] = 0;
         }
         if (pStart != NULL)
            *pStart = pi.start_time;
         if (pStop != NULL)
            *pStop = pi.stop_time;
         if (pThemes != 0)
            memcpy(pThemes, pi.pdc_themes, 7);
         if (pThemeCount != NULL)
         {
            *pThemeCount = 0;
            for (themeIdx = 0; themeIdx <= 7; themeIdx++)
            {
               if (pi.pdc_themes[themeIdx] != 0)
                  *pThemeCount++;
               else
                  break;
            }
         }
         result = TRUE;
      }
      free(pInfo);
   }
   return result;
}

static bool Plugin_SetStation( const char * pChanName, uint cni, uint inputSrc, uint freq )
{
  return WinSharedMemClient_SetStation(pChanName, -1,
                                       cni,
                                       (inputSrc == 0) /*is_tuner*/,
                                       freq,
                                       -1 /* norm */,
                                       1 /* epgPiCnt */ );
}

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


__declspec(dllexport) TVApp* InitThisPlugin()
{
	CurrentTVApp.Init			= (char (__cdecl *)(const WINSHMCLNT_TVAPP_INFO *, unsigned int, int *))WinSharedMemClient_Init;
	CurrentTVApp.Exit			= WinSharedMemClient_Exit;

	CurrentTVApp.AddPacket		= TtxDecode_AddPacket;
	CurrentTVApp.AddVpsData		= TtxDecode_AddVpsData;
	CurrentTVApp.AcqNewVbiFrame = TtxDecode_NewVbiFrame;

	CurrentTVApp.SetStation		= (char (__cdecl *)(const char *, unsigned int, unsigned int, unsigned int))Plugin_SetStation;
	CurrentTVApp.GetEpgEvent    = (int (__cdecl *)(void))WinSharedMemClient_GetEpgEvent;
	CurrentTVApp.GetProgInfo    = Plugin_GetProgInfo;
	CurrentTVApp.GetCmdArgv     = WinSharedMemClient_GetCmdArgv;
	CurrentTVApp.GetErrorMsg    = WinSharedMemClient_GetErrorMsg;

	return &CurrentTVApp;
}
